/*
   Copyright (C) 2003 Lars Lindner <lars.lindner@gmx.net>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include "conf.h"
#include "callbacks.h"
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
#include <libxml/nanohttp.h>

#define FEEDLIST		"/apps/liferea/feedlist"

#define MAX_GCONF_PATHLEN	256

GConfClient *client;

int is_gconf_error(GError *err) {

	if(err != NULL) {
		g_print(err->message);
		g_error_free(err);
		err = NULL;
		return 1;
	}
	
	return 0;
}

void loadConfig() {
	GError	*err = NULL;
        gchar	*proxy_url;
        gint	proxy_port;
        gchar	*proxy_host;
	
	// GConf client
	if (client == NULL)
		client = gconf_client_get_default();

	// Make sure the base directory exists. Subdirectories will be added 
	// as needed.
	gconf_client_add_dir(client, PATH, GCONF_CLIENT_PRELOAD_NONE, NULL);	
		
        /* load proxy settings for libxml */
        xmlNanoHTTPInit();
        
	if(getBooleanConfValue(USE_PROXY)) {
	        proxy_host = getStringConfValue(PROXY_HOST);
        	proxy_port = getNumericConfValue(PROXY_PORT);
		
        	proxy_url = g_strdup_printf("http://%s:%d/", proxy_host, proxy_port);
		g_print("using proxy: \"%s\"\n", proxy_url);
        	xmlNanoHTTPScanProxy(proxy_url);
		
		g_free(proxy_host);
		g_free(proxy_url);
	} else {
		/* this is neccessary to reset proxy after config change */
        	xmlNanoHTTPScanProxy(NULL);	
	}
}

void removeFeedFromConfig(gchar *feedkey) {
	GError		*err = NULL;
	GConfValue	*value;
	GConfValue	*element;
	GSList		*list, *newlist = NULL;
	GSList		*iter;
	const char	*tmp;
	int 		error = 0;

	/* remove key from key list */
	value = gconf_client_get(client, FEEDLIST, &err);
	is_gconf_error(err);
	list = gconf_value_get_list(value);

	iter=list;
	while(iter != NULL) {
		element = iter->data;
		tmp = gconf_value_get_string(element);
		if(0 != strcmp(tmp, feedkey)) {
			newlist = g_slist_append(newlist, element);
		}
		iter = g_slist_next(iter);		
	}
	
	/* write new list back to gconf */
	value = gconf_value_new(GCONF_VALUE_LIST);
	gconf_value_set_list_type(value, GCONF_VALUE_STRING);
	gconf_value_set_list(value, newlist);
	gconf_client_set(client, FEEDLIST, value, &err);
	is_gconf_error(err);
	g_free(value);		

	g_slist_free(list);
	g_slist_free(newlist);
}

/* return the feed list */
GSList * getFeedKeyList(void) {
	GSList		*feedlist;
	GError		*err = NULL;
	GConfValue	*value;
	
	value = gconf_client_get(client, FEEDLIST, &err);
	is_gconf_error(err);
	
	feedlist = gconf_value_get_list(value);
	g_free(value);
	
	return feedlist;
}

static void setFeedKeyList(GSList *newlist) {
	GConfValue	*value;
	GError		*err = NULL;
	
	value = gconf_value_new(GCONF_VALUE_LIST);
	gconf_value_set_list_type(value, GCONF_VALUE_STRING);
	gconf_value_set_list(value, newlist);
	gconf_client_set(client, FEEDLIST, value, &err);
	is_gconf_error(err);
	g_free(value);		
}

/* compare function for numerical GSList sorting in getFreeFeedKey() */
static gint compare_func(gconstpointer a, gconstpointer b) {
	GError		*err = NULL;
	GConfValue	*element;
	const char	*feedkey1, *feedkey2;	
	gint		result, res;
	
	element = (GConfValue *)a;
	feedkey1 = gconf_value_get_string(element);
	is_gconf_error(err);
	
	element = (GConfValue *)b;
	feedkey2 = gconf_value_get_string(element);
	is_gconf_error(err);	
	
	result = atoi(feedkey1) - atoi(feedkey2);
	res = (result)?result/abs(result):0;

	return res;
}

/* this method is used to get a feedkey which is not already
   used for an existing feed */
gchar * getFreeFeedKey(void) {
	GError		*err = NULL;
	GSList		*iter = NULL;	
	GConfValue	*element;
	gchar		*gconfpath;
	gchar		*url;
	const char 	*feedkey;
	int 		i;

	/* actually we use only numbers as feedkeys... */
	int		nr = 1;
	gchar		*newfeedkey;	/* the number as a string */
	newfeedkey = g_strdup_printf("%d", nr);

	iter = getFeedKeyList();
	iter = g_slist_sort(iter, compare_func);	
	while (iter != NULL) {
		element = iter->data;
		feedkey = gconf_value_get_string(element);
		is_gconf_error(err);	
		
		g_assert(NULL != feedkey);			

		if(0 == strcmp(feedkey, newfeedkey)) {
			nr++;
			g_free(newfeedkey);
			newfeedkey = g_strdup_printf("%d", nr);			
		}
	
		iter = g_slist_next(iter);
	}
	
	g_print("new feed key: \"%s\"\n", newfeedkey);
	return newfeedkey;
}

/* adds the given URL to the list of feeds... */
int addFeedToConfig(gchar *feedkey, gchar *url) {
	GError		*err = NULL;
	GConfValue	*newkey = NULL;
	GSList		*list, *newlist;
	int 		error = 0;

	/* save feed url */
	if(0 == setFeedURLInConfig(feedkey, url)) {
	
		/* add feedkey to feedlist */
		list = getFeedKeyList();
		
		newkey = gconf_value_new(GCONF_VALUE_STRING);
		gconf_value_set_string(newkey, feedkey);
		is_gconf_error(err);
		
		newlist = g_slist_append(list, newkey);
		
		/* write new list back to gconf */
		setFeedKeyList(newlist);
		
		g_free(newkey);
		g_slist_free(newlist);
	}
}

int setFeedTitleInConfig(gchar *feedkey, gchar *feedname) {
	GError		*err = NULL;
	gchar		*gconfpath;
					
	/* update title */
	gconfpath = g_strdup_printf("%s/%s/name", PATH, feedkey);
	gconf_client_set_string(client, gconfpath, feedname, &err);
	g_free(gconfpath);	
	if(is_gconf_error(err))
		return 1;
	
	return 0;	
}

int setFeedURLInConfig(gchar *feedkey, gchar *feedurl) {
	GError		*err = NULL;
	gchar		*gconfpath;
					
	/* update URL */
	gconfpath = g_strdup_printf("%s/%s/url", PATH, feedkey);
	gconf_client_set_string(client, gconfpath, feedurl, &err);
	g_free(gconfpath);	
	if(is_gconf_error(err))
		return 1;
	
	return 0;	
}

void moveUpFeedPositionInConfig(gchar *feedkey) {
	GError		*err = NULL;
	GSList		*iter, *list;
	GConfValue	*element;
	const char	*actualkey;
	gboolean	found = FALSE;
	gint 		pos = 0;
	
	list = iter = getFeedKeyList();
	/* scan the list for the element to move and unlink it */
	while(iter != NULL) {
		element = iter->data;
		actualkey = gconf_value_get_string(element);
		is_gconf_error(err);		
	
		if(0 == strcmp(feedkey, actualkey)) {
			list = g_slist_remove_link(list, iter);
			found = 1;
			break;
		}

		iter = g_slist_next(iter);
		pos ++;
	}
	
	if(found) {
		/* reinsert at next position */
		element = gconf_value_new(GCONF_VALUE_STRING);
		gconf_value_set_string(element, feedkey);
		is_gconf_error(err);
		
		list = g_slist_insert(list, element, pos - 1);

		/* save it back to gconf */
		setFeedKeyList(list);
	} else {
		g_error("internal error, element should be moved but was not found in list!\n");
	}
	
}

void moveDownFeedPositionInConfig(gchar *feedkey) {
	GError		*err = NULL;
	GSList		*iter, *list;
	GConfValue	*element;
	const char	*actualkey;
	gboolean	found = FALSE;
	gint 		pos = 0;
	
	list = iter = getFeedKeyList();
	/* scan the list for the element to move and unlink it */
	while(iter != NULL) {
		element = iter->data;
		actualkey = gconf_value_get_string(element);
		is_gconf_error(err);		
	
		if(0 == strcmp(feedkey, actualkey)) {
			list = g_slist_remove_link(list, iter);
			found = 1;
			break;
		}

		iter = g_slist_next(iter);
		pos ++;
	}
	
	if(found) {
		/* reinsert at next position */
		element = gconf_value_new(GCONF_VALUE_STRING);
		gconf_value_set_string(element, feedkey);
		is_gconf_error(err);
		
		list = g_slist_insert(list, element, pos + 1);

		/* save it back to gconf */
		setFeedKeyList(list);
	} else {
		g_error("internal error, element should be moved but was not found in list!\n");
	}
	
}

void sortFeedKeyList(void) {
}

void loadFeeds(void) {
	GError		*err = NULL;
	GSList		*iter = NULL;
	GConfValue	*element, *value;
	gchar		*gconfpath;
	gchar		*feedname;
	const char	*feedkey;
	gchar		*url;

	value = gconf_client_get(client, FEEDLIST, &err);
	is_gconf_error(err);
	
	if(NULL == value) {
		/* first call key still does not exists, we create
		   an empty list */
		value = gconf_value_new(GCONF_VALUE_LIST);
		gconf_value_set_list_type(value, GCONF_VALUE_STRING);
		gconf_value_set_list(value, NULL);
		gconf_client_set(client, FEEDLIST, value, &err);
		is_gconf_error(err);
	}
	
	if(GCONF_VALUE_LIST != value->type) {
		g_print("fatal thats not a list type!\n");
		return;
	}
	iter = gconf_value_get_list(value);
	
	while (iter != NULL) {
		element = iter->data;
		feedkey = gconf_value_get_string(element);
		is_gconf_error(err);		
		g_assert(NULL != feedkey);			

		gconfpath = g_strdup_printf("%s/%s/url", PATH, feedkey);
		url = gconf_client_get_string(client, gconfpath, &err);
		is_gconf_error(err);
		g_free(gconfpath);

		gconfpath = g_strdup_printf("%s/%s/name", PATH, feedkey);
		feedname = gconf_client_get_string(client, gconfpath, &err);
		is_gconf_error(err);
		g_free(gconfpath);		

		newFeed((gchar *)feedkey, url, TRUE, FALSE);
		setFeedTitle(NULL, feedname);

		iter = g_slist_next(iter);
	}
}

/* generic configuration access methods */
void setBooleanConfValue(gchar *valuename, gboolean value) {
	GError		*err = NULL;
	GConfValue	*gcv;
	
	g_assert(valuename != NULL);
	
	gcv = gconf_value_new(GCONF_VALUE_BOOL);
	gconf_value_set_bool(gcv, value);
	gconf_client_set(client, valuename, gcv, &err);
	is_gconf_error(err);
}

gboolean getBooleanConfValue(gchar *valuename) {
	GError		*err = NULL;
	gboolean	value;

	g_assert(valuename != NULL);
		
	value = gconf_client_get_bool(client, valuename, &err);	
	is_gconf_error(err);
		
	return value;
}
void setStringConfValue(gchar *valuename, gchar *value) {
	GError		*err = NULL;
	GConfValue	*gcv;
	
	g_assert(valuename != NULL);
	
	gcv = gconf_value_new(GCONF_VALUE_STRING);
	gconf_value_set_string(gcv, value);
	gconf_client_set(client, valuename, gcv, &err);
	is_gconf_error(err);
}

gchar * getStringConfValue(gchar *valuename) {
	GError		*err = NULL;
	gchar		*value;

	g_assert(valuename != NULL);
		
	value = gconf_client_get_string(client, valuename, &err);	
	is_gconf_error(err);
		
	return value;
}

void setNumericConfValue(gchar *valuename, gint value) {
	GError		*err = NULL;
	GConfValue	*gcv;
	
	g_assert(valuename != NULL);
	
	gcv = gconf_value_new(GCONF_VALUE_INT);
	gconf_value_set_int(gcv, value);
	gconf_client_set(client, valuename, gcv, &err);
	is_gconf_error(err);
}

gint getNumericConfValue(gchar *valuename) {
	GError		*err = NULL;
	gint		value;

	g_assert(valuename != NULL);
		
	value = gconf_client_get_int(client, valuename, &err);	
	is_gconf_error(err);
		
	return value;
}
